#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.GeneticProgramming.GlucosePrediction {
  [StorableClass]
  [Item(CurvedInsVariableSymbol.Name, "")]
  public class CurvedInsVariableSymbol : Symbol {
    public new const string Name = "CurvedInsVariableSymbol";
    #region Properties

    [Storable]
    private double minAlpha;
    public double MinAlpha {
      get { return minAlpha; }
      set {
        if (value != minAlpha) {
          minAlpha = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }
    [Storable]
    private double maxAlpha;
    public double MaxAlpha {
      get { return maxAlpha; }
      set {
        if (value != maxAlpha) {
          maxAlpha = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }

    [Storable]
    private double minBeta;
    public double MinBeta {
      get { return minBeta; }
      set {
        if (value != minBeta) {
          minBeta = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }

    [Storable]
    private double maxBeta;
    public double MaxBeta {
      get { return maxBeta; }
      set {
        if (value != maxBeta) {
          maxBeta = value;
          OnChanged(EventArgs.Empty);
        }
      }
    }

    private const int minimumArity = 0;
    private const int maximumArity = 0;

    public override int MinimumArity {
      get { return minimumArity; }
    }

    public override int MaximumArity {
      get { return maximumArity; }
    }
    #endregion

    [StorableHook(HookType.AfterDeserialization)]
    private void AfterDeserialization() {
    }

    [StorableConstructor]
    protected CurvedInsVariableSymbol(bool deserializing)
      : base(deserializing) {
    }

    protected CurvedInsVariableSymbol(CurvedInsVariableSymbol original, Cloner cloner)
      : base(original, cloner) {
      minAlpha = original.minAlpha;
      maxAlpha = original.maxAlpha;
      minBeta = original.minBeta;
      maxBeta = original.maxBeta;
    }

    public CurvedInsVariableSymbol(string name, string desc, double minAlpha = 0.001, double maxAlpha = 0.999, double minBeta = 0.001, double maxBeta = 0.999)
      : base(name, desc) {
      this.minAlpha = minAlpha;
      this.maxAlpha = maxAlpha;
      this.minBeta = minBeta;
      this.maxBeta = maxBeta;
    }

    public override ISymbolicExpressionTreeNode CreateTreeNode() {
      return new CurvedInsVariableTreeNode(this);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new CurvedInsVariableSymbol(this, cloner);
    }
  }
}
